/* REPORTS */
CREATE TABLE tblReports (
	id INTEGER PRIMARY KEY AUTOINCREMENT, 
	informerURI TEXT, -- uri of informer who made this report
	target TEXT, -- what this report is about, may include wildcards
	link TEXT, -- uri of representative page
	isInformerStatement TEXT, -- boolen is the target (URI) an informer statement?
	quality TEXT,
	date TEXT,
	description TEXT
	--tags (in tblTags)
);

CREATE INDEX ndxReports_informerURI
ON tblReports (informerURI);

CREATE INDEX ndxReports_target
ON tblReports (target);

/* I remember being told that it's bad to index a boolean value, but it seems like it
would be useful here, to help the query to zoom in on just the informer reports without
having to do a table scan */
CREATE INDEX ndxReports_isInformerStatement
ON tblReports (isInformerStatement);

/* TAGS */

CREATE TABLE tblTags (
	tag TEXT,
	degree INTEGER, -- NYI: degree of tag, eg "20% politics, 40% funny"
	reportID_ref INTEGER -- id of report that gave this tag
);

CREATE INDEX ndxTags_tag
ON tblTags (tag);

CREATE INDEX ndxTags_reportID_ref
ON tblTags (reportID_ref);


/* INFORMER */

CREATE TABLE tblInformerProfiles (
	informerURI PRIMARY KEY,
	title TEXT,
	uri TEXT,
	link TEXT,
	description TEXT,
	image TEXT,
	pubKey TEXT, --NYI
	maxPing INTEGER  -- NYI how often the statement may be accessed (in...milliseconds??)
	--tags (in tblTags)  -- self-described tags
);

CREATE TABLE tblInformerInfo (
	informerURI PRIMARY KEY,
	hops INTEGER,
	failDate TEXT,
	visitDate TEXT,
	statementModifiedDate TEXT
);

