"""
PHP date() style date formatting
See http://www.php.net/date for format strings

Usage:
>>> import datetime
>>> d = datetime.datetime.now()
>>> df = DateFormat(d)
>>> print df.format('jS F Y H:i')
7th October 2003 11:39
>>>

"""

# Simon Willison, 7th October 2003
# License = the same as Python

class NotImplemented(Exception):
    pass

class DateFormat:
    weekdays = 'Monday Tuesday Wednesday Thursday Friday Saturday Sunday'.split()
    months = 'January February March April May June July August September October November December'.split()
    def __init__(self, d):
        self.date = d
    def a(self):
        '"am" or "pm"'
        if self.date.hour > 11:
            return 'pm'
        else:
            return 'am'
    def A(self):
        '"AM" or "PM"'
        return self.a().upper()
    def B(self):
        "Swatch Internet time"
        raise NotImplemented
    def d(self):
        'day of the month, 2 digits with leading zeros; i.e. "01" to "31"'
        return '%02d' % self.date.day
    def D(self):
        'day of the week, textual, 3 letters; e.g. "Fri"'
        return self.weekdays[self.date.weekday()][0:3]
    def F(self):
        'month, textual, long; e.g. "January"'
        return self.months[self.date.month - 1]
    def g(self):
        'hour, 12-hour format without leading zeros; i.e. "1" to "12"'
        if self.date.hour == 0:
            return 12
        if self.date.hour > 12:
            return self.date.hour - 12
        return self.date.hour
    def G(self):
        'hour, 24-hour format without leading zeros; i.e. "0" to "23"'
        return self.date.hour
    def h(self):
        'hour, 12-hour format; i.e. "01" to "12"'
        return '%02d' % self.g()
    def H(self):
        'hour, 24-hour format; i.e. "00" to "23"'
        return '%02d' % self.G()
    def i(self):
        'minutes; i.e. "00" to "59"'
        return '%02d' % self.date.minute
    def I(self):
        '"1" if Daylight Savings Time, "0" otherwise.'
        raise NotImplemented
    def j(self):
        'day of the month without leading zeros; i.e. "1" to "31"'
        return self.date.day
    def l(self):
        'day of the week, textual, long; e.g. "Friday"'
        return self.weekdays[self.date.weekday()]
    def L(self):
        'boolean for whether it is a leap year; i.e. "0" or "1"'
        raise NotImplemented
    def m(self):
        'month; i.e. "01" to "12"'
        return '%02d' % self.date.month
    def M(self):
        'month, textual, 3 letters; e.g. "Jan"'
        return self.months[self.date.month - 1][0:3]
    def n(self):
        'month without leading zeros; i.e. "1" to "12"'
        return self.date.month
    def O(self):
        'Difference to Greenwich time in hours; e.g. "+0200"'
        raise NotImplemented
    def r(self):
        'RFC 822 formatted date; e.g. "Thu, 21 Dec 2000 16:01:07 +0200"'
        raise NotImplemented
    def s(self):
        'seconds; i.e. "00" to "59"'
        return '%02d' % self.date.second
    def S(self):
        'English ordinal suffix for the day of the month, 2 characters; i.e. "st", "nd", "rd" or "th"'
        if self.date.day in (11, 12): # Special case
            return 'th'
        last = self.date.day % 10
        if last == 1:
            return 'st'
        if last == 2:
            return 'nd'
        if last == 3:
            return 'rd'
        return 'th'
    def t(self):
        'number of days in the given month; i.e. "28" to "31"'
        raise NotImplemented
#    def T(self):
#        'Timezone setting of this machine; e.g. "EST" or "MDT"'
#        raise NotImplemented
    def U(self):
        'seconds since the Unix Epoch (January 1 1970 00:00:00 GMT)'
        raise NotImplemented
    def w(self):
        'day of the week, numeric, i.e. "0" (Sunday) to "6" (Saturday)'
        weekday = self.date.weekday()
        if weekday == 0:
            return 6
        return weekday - 1
    def W(self):
        'ISO-8601 week number of year, weeks starting on Monday'
        raise NotImplemented
    def Y(self):
        'year, 4 digits; e.g. "1999"'
        return self.date.year
    def y(self):
        'year, 2 digits; e.g. "99"'
        return str(self.date.year)[2:]
    def z(self):
        'day of the year; i.e. "0" to "365"'
        raise NotImplemented
#    def Z(self):
#        'timezone offset in seconds (i.e. "-43200" to "43200"). The offset for timezones west of UTC is always negative, and for those east of UTC is always positive.'
#        raise NotImplemented
    def format(self, formatstr):
        result = ''
        for char in formatstr:
            try:
                result += str(getattr(self, char)())
            except AttributeError:
                result += char
        return result
