# Copyright 2005 Stan James

import sys, cgi, string, time
import apsw # Python-SQLite interface
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
import os.path
import re, urllib
import codecs # needed for unicode stuff

# mddb modules
import mddb_reset
import mddb_lookupURI
import mddb_addReport
import mddb_deleteReport
import mddb_randomURI
import mddb_recentActivity
import mddb_myInformers
import mddb_myReports
import mddb_nextSyncInformer
import mddb_setUserInfo
import mddb_getUserInfo
import mddb_getStatement
import mddb_setStatement
import mddb_getChain
import mddb_searchTags
import mddb_randomURIset

global false, true
true = 1
false = 0

# http://docs.python.org/lib/module-BaseHTTPServer.html
serverVars = {}

db_setup_filename = "sql/outfoxed_db_setup.sql"

def parse_call(path, data):
	
	s = re.split('\?', path, 1)	
	if len(s) == 1:
		command=s[0]
		argpairs_raw=[]
	elif len(s) > 2:
		return "<error/>"		
	else:
		(command,argpairs_raw) = s
	
	#breakpoint = path.find("?")
	#if breakpoint == -1:
	#	return "<error/>"		
	#else:
	#	command = path[:breakpoint]
	#	argpairs_raw = path[(breakpoint+1):]
	
	params = []	
	if (len(argpairs_raw)>0):
		argpairs = re.split('&',argpairs_raw)	
		for argpair in argpairs:
			s = re.split('=',argpair)
			if len(s) != 2:
				continue
			(param,value) = s
			param = urllib.unquote(param)
			value = urllib.unquote(value)
			params.append( (param,value) )
			
	print "___MDDB:command:"+command
	if command=="/hello":
		return "hello"
	if command=="/quit":
		sys.exit(1)
		# TODO: this doesn't work. The exception raised by exit() gets caught by HTTPserver and ignored...
	elif command=="/getChain":
		return mddb_getChain.getChain                (params, cursor, serverVars)
	elif command=="/reset":
		return mddb_reset.reset                      (params, cursor, serverVars)
	elif command=="/lookupURIs":
		return mddb_lookupURI.lookupURI              (params, cursor, serverVars)
	elif command=="/searchTags":
		return mddb_searchTags.searchTags            (params, cursor, serverVars)
	elif command=="/addReport":
		return mddb_addReport.addReport              (params, cursor, serverVars)
	elif command=="/deleteReport":
		return mddb_deleteReport.deleteReport        (params, cursor, serverVars)
	elif command=="/randomURI":
		return mddb_randomURI.randomURI              (params, cursor, serverVars)
	elif command=="/randomURIset":
		return mddb_randomURIset.randomURIset        (params, cursor, serverVars)
	elif command=="/recentActivity":
		return mddb_recentActivity.recentActivity    (params, cursor, serverVars)
	elif command=="/myInformers":
		return mddb_myInformers.myInformers          (params, cursor, serverVars)
	elif command=="/myReports":
		return mddb_myReports.myReports              (params, cursor, serverVars)
	elif command=="/setUserInfo":
		return mddb_setUserInfo.setUserInfo          (params, cursor, serverVars)
	elif command=="/getUserInfo":
		return mddb_getUserInfo.getUserInfo          (params, cursor, serverVars)
	elif command=="/nextSyncInformer":
		return mddb_nextSyncInformer.nextSyncInformer(params, cursor, serverVars)
	elif command=="/setStatement":
		return mddb_setStatement.setStatement        (params, cursor, serverVars, data)
	elif command=="/getStatement":
		return mddb_getStatement.getStatement        (params, cursor, serverVars, data)
	else:	
		return("Nothing found")
		print "___MDDB:error"

# http://docs.python.org/lib/module-BaseHTTPServer.html
class MDDB_request_handler(BaseHTTPRequestHandler):

	def do_GET(self):
		if self.client_address[0] != "127.0.0.1":
			return
		self.send_response(200)
		self.send_header('Content-type', 'text/xml')
		self.end_headers()
		out = parse_call(self.path,"")
		print "type(out)",type(out)
		self.wfile.write( out.encode('utf8','replace'))
		print "___MDDB:wrote response"
		print serverVars["userURI"]
		print "___MDDB:-----------------------------------------"
				
	def do_POST(self):
		if self.client_address[0] != "127.0.0.1":	
			return;
		if (self.headers.getheader('content-type') == "text/xml"):
			data = self.rfile.read( int(self.headers.getheader('content-length')) )
		self.send_response(200)
		self.end_headers()
		self.wfile.write( parse_call(self.path,data).encode('utf8'))


class myHTTPServer (HTTPServer):
	def handle_error(self, request, client_address):
		# if we're asked to quit, then quit. 
		# TODO: maybe compress the DB at this point?
		# TODO: later, for security, this (and all other funcs) should be password (or otherwise) protected
		if str(sys.exc_type) == "exceptions.SystemExit":
			sys.exit(1)			
		HTTPServer.handle_error(self, request, client_address)



def start(port):
	try:
		server = myHTTPServer(('', port), MDDB_request_handler)
		print 'Outfoxed MDDB server is listening on port ' + repr(port)
		server.serve_forever()
	except KeyboardInterrupt:
		server.socket.close()




# Command line directives
	
print "___MDDB:Command line vars are:", sys.argv

if len(sys.argv)>1:
	serverVars["base_path"] = sys.argv[1]
	serverVars["port"] = int(sys.argv[2])
	serverVars["db_filename"] = serverVars["base_path"]+sys.argv[3]
else:
	serverVars["base_path"]= ""
	serverVars["port"] = 63047 
	serverVars["db_filename"] = serverVars["base_path"]+"outfoxed_mddb"

# see if server is already running
try:
	serverResponse =  urllib.urlopen("http://127.0.0.1:"+repr(serverVars["port"])+"/hello").read()
	isRunning = true
except IOError:
	isRunning = false


if isRunning:
	print "___MDDB:Server is already running. This process will exit in 3 seconds."
	time.sleep(3)
	sys.exit(1)
else:
	#DBexists = os.path.isfile(serverVars["db_filename"])
		
	# Opening/creating database
	connection=apsw.Connection(serverVars["db_filename"])
	cursor=connection.cursor()

	DBexists = false
	try:
		for x in cursor.execute("SELECT tbl_name FROM sqlite_master WHERE tbl_name='tblSystemVars';"):
			DBexists = x[0]
	except:
		DBexists = false
		
	if not(DBexists):
		# create db
		print "___MDDB:Database is not set up. Creating from scratch."
		db_setup_string = open(serverVars["base_path"]+db_setup_filename).read()
		try:
			cursor.execute(db_setup_string)
		except apsw.MisuseError: #TODO: WTF is this non-error?
			print "___MDDB:stupid apsw.MisuseError"	

	for x in cursor.execute("SELECT tbl_name FROM sqlite_master WHERE tbl_name='tblSystemVars';"):
		DBexists = x[0]

	if not(DBexists):
		print "___MDDB:Could not create database. Exiting."
		sys.exit(-1)
		
	if DBexists:
		print "___MDDB:Database file found :",serverVars["db_filename"]
		gUserURI = ""
		for x in cursor.execute("SELECT varValue FROM tblSystemVars WHERE varName='userURI';"):
			gUserURI = x[0]
		print "___MDDB:User is " + gUserURI
		serverVars["userURI"] = gUserURI
		
	if gUserURI:
		# Go!!	
		start(serverVars["port"])
	else:
		print "___MDDB:No USER URI found. Exiting."


