/*
Hops / Chain Calulator
Copyright 2005 Stanley James
*/

BEGIN; 

-- Set hops to default high number
UPDATE tblInformerProfiles
SET hops = 999
;

DELETE FROM tblChain;

INSERT INTO tblChain (hops,i1)
SELECT 
	1, R1.link
FROM
	tblReports R1
WHERE
	R1.isInformerStatement = 1
	AND R1.informer = '_USER_URI_'
	-- no loops
	AND R1.link <> '_USER_URI_' -- user isn't reporting on himself
;

INSERT INTO tblChain (hops,i1,i2)
SELECT DISTINCT
	2, R1.link, C.i1
FROM
	tblReports R1, tblChain C
WHERE
	R1.isInformerStatement = 1
  AND C.i1 = R1.informer
  AND C.hops = 1
  -- no loops
  AND R1.link <> '_USER_URI_'
  AND R1.link <> C.i1
  AND R1.link NOT IN 
  (
  	SELECT i1
  	FROM tblChain
  )
;

INSERT INTO tblChain (hops,i1,i2, i3)
SELECT DISTINCT
	3, R1.link, C.i1, C.i2
FROM
	tblReports R1, tblChain C
WHERE
	R1.isInformerStatement = 1
  AND C.i1 = R1.informer
  AND C.hops = 2  
  -- no loops
  AND R1.link <> '_USER_URI_'
  AND R1.link <> C.i1
  AND R1.link <> C.i2
  AND R1.link NOT IN 
  (
  	SELECT i1
  	FROM tblChain
  )
  
;

INSERT INTO tblChain (hops,i1,i2, i3, i4)
SELECT DISTINCT
	4, R1.link, C.i1, C.i2, C.i3
FROM
	tblReports R1, tblChain C
WHERE
	R1.isInformerStatement = 1
  AND C.i1 = R1.informer
  AND C.hops = 3  
  -- no loops
  AND R1.link <> '_USER_URI_'
  AND R1.link <> C.i1
  AND R1.link <> C.i2
  AND R1.link <> C.i3
  AND R1.link NOT IN 
  (
  	SELECT i1
  	FROM tblChain
  )
  
;

INSERT INTO tblChain 
	(hops, i1, i2, i3, i4, i5)
SELECT DISTINCT
	5, R1.link, C.i1, C.i2, C.i3, C.i4
FROM
	tblReports R1, tblChain C
WHERE
	R1.isInformerStatement = 1
  AND C.i1 = R1.informer
  AND C.hops = 4  
  -- no loops
  AND R1.link <> '_USER_URI_'
  AND R1.link <> C.i1
  AND R1.link <> C.i2
  AND R1.link <> C.i3
  AND R1.link <> C.i4
  AND R1.link NOT IN 
  (
  	SELECT i1
  	FROM tblChain
  )
;

-- UNnormalized. Copy hop count into profile table.
UPDATE tblInformerProfiles
SET hops = (
	SELECT MIN(hops) FROM tblChain WHERE i1 = uri
)
;
-- user is always at zero hops, even though they have no reports on them.
UPDATE tblInformerProfiles
SET hops = 0
WHERE uri = '_USER_URI_'
;
-- "orphan" informers (with no good reports on them) get hops set to 999, and later deleted.
UPDATE tblInformerProfiles
SET hops = 999
WHERE hops IS NULL
;

-- TODO: we need a trigger on tblInformerProfiles (???)

-- P1 is not trusted by P2, and P2 is more trusted by user
UPDATE tblInformerProfiles
SET hops = 999
WHERE uri IN 
(
	SELECT P1.uri --, P2.uri, R.quality, R.description
	FROM 
		tblInformerProfiles P1, tblInformerProfiles P2, tblReports R
	WHERE 
		R.link = P1.uri
		AND R.informer = P2.uri
		AND R.quality < 0
		AND P2.hops < P1.hops
)
;

DELETE FROM tblReports
WHERE informer IN (SELECT uri FROM tblInformerProfiles WHERE hops=999);

DELETE FROM tblTags
WHERE informer IN (SELECT uri FROM tblInformerProfiles WHERE hops=999);

DELETE FROM tblInformerDownload
WHERE informer IN (SELECT uri FROM tblInformerProfiles WHERE hops=999);

DELETE FROM tblChain
WHERE i1 IN (SELECT uri FROM tblInformerProfiles WHERE hops=999);

DELETE FROM tblInformerProfiles WHERE hops=999;


COMMIT;