/* Copyright 2005 Stanley James */
BEGIN
;

/* SYSTEM VARS */
CREATE TABLE tblSystemVars (
	varName TEXT PRIMARY KEY,
	varValue TEXT
)
;

/* INFORMER */
CREATE TABLE tblInformerProfiles (
	-- these columns are defined by the statement
	uri TEXT PRIMARY KEY, -- uri of informer statement
	name TEXT,
	link TEXT,
	description TEXT,
	image TEXT,
	pubKey TEXT, --NYI
	maxPing INTEGER,  -- NYI how often the statement may be accessed (in...milliseconds??)
	tags,  -- self-described tags (as these are self-described, and just recommendations at that, they are not included in index)
	
	-- these columns are for internal management
	hops INTEGER
)
;

CREATE INDEX ndxInformerProfiles_hops
ON tblInformerProfiles (hops)
;

/* REPORTS */
CREATE TABLE tblReports (
	informer TEXT, -- uri of informer who made this report
	link TEXT, -- uri (due to rdf restrictions, this is the effective "primary key" for the report, per informer)
	scope TEXT, -- where this report applies, may include * wildcards (if empty, report applys to link only)
	title TEXT, -- title of the page (or name of informer, or plain name of file, or...)
	isInformerStatement TEXT, -- is this a statement? (ie, should we attempt to download from here?)
	quality TEXT,
	date TEXT,
	description TEXT,
	tags TEXT --(UN-normalized. real data in tblTags)
)
;

CREATE INDEX ndxReports_informer
ON tblReports (informer)
;

CREATE INDEX ndxReports_link
ON tblReports (link)
;

/* I remember being told that it's bad to index a boolean value, but it seems like it
would be useful here, to help the query zoom in on just the informer reports without
having to do a table scan. */
CREATE INDEX ndxReports_isInformerStatement
ON tblReports (isInformerStatement)
;

/* TAGS */

CREATE TABLE tblTags (
	tag TEXT,
	degree INTEGER, -- NYI: degree of tag, eg "20% politics, 40% funny"
	informer TEXT, -- uri of informer who made this tag
	uri TEXT -- uri that this tag was applied to by the informer
)
;

CREATE INDEX ndxTags_tag
ON tblTags (tag)
;

CREATE INDEX ndxTags_uri
ON tblTags (uri)
;

CREATE INDEX ndxTags_informer
ON tblTags (informer)
;

/* INFORMER download stuff */
CREATE TABLE tblInformerDownload (
	informer TEXT PRIMARY KEY, -- uri of informer [statement]
	visitDate TEXT, -- date when we last attempted to download
	modifiedDate TEXT, -- date when statement was last changed (as reported by the server...used to detect changes)
	failDate TEXT, -- date we first failed to download, with no subsequent success.  Empty if last download attempt was successful
	comment TEXT -- any error string generated during the last download attempt
)
;

CREATE INDEX ndxtblInformerDownload_visitDate
ON tblInformerDownload (visitDate)
;

/* Visited URI's (For random page function */

CREATE TABLE tblVisitedURIs (
  URI TEXT PRIMARY KEY,
  visitDate TEXT
)
;

/* Chains from informers to user (from i1 to user) UN-normalized*/

CREATE TABLE tblChain (
	hops INTEGER,
	i1 TEXT,
	i2 TEXT,
	i3 TEXT,
	i4 TEXT,
	i5 TEXT
)
;

/* New user default values */

INSERT INTO "tblSystemVars" VALUES('userURI', 'chrome://suzanne/content/localuser.html')
;
INSERT INTO "tblInformerProfiles" VALUES('chrome://suzanne/content/localuser.html', 'Anonymous', '', '', 'http://getoutfoxed.com/files/default.jpg', '', 0, '', 0)
;
INSERT INTO "tblReports" VALUES('chrome://suzanne/content/localuser.html', 'http://getoutfoxed.com/sample/dangerous', '', 'Sample Dangerous Page', '0', '-50', '2005-04-02T18:17:24Z', 'Sample Dangerous Page', 'sample')
;
INSERT INTO "tblReports" VALUES('chrome://suzanne/content/localuser.html', 'http://getoutfoxed.com/sample/good', '', 'Sample Good Page', '0', '50', '2005-04-02T18:17:52Z', 'Sample Good Page', 'sample')
;
INSERT INTO "tblReports" VALUES('chrome://suzanne/content/localuser.html', 'http://getoutfoxed.com/sample/bad', '', 'Sample Bad Page', '0', '-25', '2005-04-02T18:18:11Z', 'Sample bad page.', 'sample')
;
COMMIT
;


