<?xml version="1.0" encoding="utf-8"?>

<xsl:stylesheet 
	xmlns:rss="http://purl.org/rss/1.0/" 
	xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
	xmlns:taxo="http://purl.org/rss/1.0/modules/taxonomy/"
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:of="http://trustbase.org/quality/0.1/"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	version="1.0"
>

	<xsl:variable name="singquote">'</xsl:variable> <!-- HACK: we need better single quote handling -->
	
	<xsl:output method="text" indent="yes"/>

	<!-- override built-in template for text/atts -->
	<xsl:template match="text()|@*"/>

	<xsl:template match="rdf:RDF">
		BEGIN;
		<xsl:apply-templates/>
		COMMIT;
	</xsl:template>
	
	<xsl:template match="rss:channel">
		/* Clear out the existing report */

		DELETE FROM tblInformerProfiles
		WHERE uri = '_INFORMER_URI_';

		DELETE FROM tblReports
		WHERE informer = '_INFORMER_URI_';

		DELETE FROM tblTags
		WHERE informer = '_INFORMER_URI_';

		INSERT INTO tblInformerProfiles 
			( uri, name, link, description, image, pubKey, maxPing, tags)
		VALUES
		(
			'_INFORMER_URI_',
			'<xsl:value-of select="rss:title"/>',
			'<xsl:value-of select="rss:link"/>',
			'<xsl:value-of select="translate(rss:description,$singquote,'~')"/>',
			'<xsl:value-of select="rss:image/@rdf:resource"/>',
			'<xsl:value-of select="of:pubKey"/>',
			'<xsl:value-of select="of:maxPing"/>',
			'<xsl:value-of select="dc:subject"/>'			
		);
		<xsl:apply-templates/>
	</xsl:template>

	
	<xsl:template match="rss:item">
		INSERT INTO tblReports
			(informer,scope,link,isInformerStatement,quality,date,description,tags)
		VALUES
		(
			'_INFORMER_URI_',
			'<xsl:value-of select="of:scope"/>',
			'<xsl:value-of select="rss:link"/>',
			'<xsl:if test="contains(dc:subject, '_RSS')">1</xsl:if>',
			'<xsl:if test="not(of:quality)">2</xsl:if><xsl:value-of select="of:quality"/>',<!-- Give nominal positive rating for items with no quality tag-->
			'<xsl:value-of select="dc:date"/>',
			'<xsl:value-of select="translate(rss:description,$singquote,'~')"/>',
			'<xsl:value-of select="dc:subject"/>'
		);			
			
		<xsl:if test="normalize-space(dc:subject) != ''">
			<xsl:apply-templates select="dc:subject" mode="doTags" >
				<xsl:with-param name="degree" select="100" />
				<xsl:with-param name="informer" select="'_INFORMER_URI_'" />
				<xsl:with-param name="uri" select="rss:link" />
			</xsl:apply-templates>
		</xsl:if>
	</xsl:template>
	
	<!-- http://www.biglist.com/lists/xsl-list/archives/200109/msg00507.html -->
	<xsl:template match="node()" mode="doTags" name="doTags">
	  <xsl:param name="string" select="." />
	  <xsl:param name="degree" select="100" />
	  <xsl:param name="informer" />
	  <xsl:param name="uri" />
	  
	  <xsl:choose>
	  	<xsl:when test="normalize-space($string) = ''">
	  	<!-- if string is empty, do nothing -->
	  	</xsl:when>
	  
	    <xsl:when test="contains($string, ' ')">
	    <!-- if the string contains a space -->
			INSERT INTO tblTags 
		 		(tag,degree,informer,uri)
			VALUES
			(
				'<xsl:value-of select="substring-before($string, ' ')" />',
				<xsl:value-of select="$degree"/>,
				'<xsl:value-of select="$informer"/>',
				'<xsl:value-of select="$uri"/>'
			);
	    	
	      <!-- and then call the template recursively on the rest of the string -->
	      <xsl:call-template name="doTags">
	        <xsl:with-param name="string" select="substring-after($string, ' ')" />
	        <xsl:with-param name="degree" select="$degree" />
	        <xsl:with-param name="informer" select="$informer" />
	        <xsl:with-param name="uri" select="$uri" />
	      </xsl:call-template>
	    </xsl:when>
	    
	    <xsl:otherwise>
	    <!-- if the string doesn't contain a space, just give the tag -->
			INSERT INTO tblTags 
				(tag,degree,informer,uri)
			VALUES
			(
				'<xsl:value-of select="$string" />',
				<xsl:value-of select="$degree"/>,
				'<xsl:value-of select="$informer"/>',
				'<xsl:value-of select="$uri"/>'
			);	      
	    </xsl:otherwise>
	  </xsl:choose>
	</xsl:template>

	
</xsl:stylesheet>